Package: PBSmodelling
Version: 2.70.2
Date: 2025-11-13
Title: GUI Tools Made Easy: Interact with Models and Explore Data
Authors@R: c(
   person(c("Jon","T."),"Schnute",role="aut",email="schnutej@shaw.ca"),
   person("Alex","Couture-Beil",role="aut",email="alex@mofo.ca"),
   person("Rowan","Haigh",role=c("aut"),email="rowan.haigh@dfo-mpo.gc.ca"),
   person("Nicholas","Boers",role="aut",email="nboers@gmail.com"),
   person("Anisa","Egeli",role="aut",email="aegeli@gmail.com"),
   person(c("A.","R."),"Kronlund",role="ctb",email="interfacefisheries@gmail.com"),
   person("Steve","Martell",role="ctb",email="martell.steve@gmail.com"),
   person("Norm","Olsen",role="ctb",email="norm.olsen@dfo-mpo.gc.ca"),
   person("Nick", "Fisch", role = "cre", email = "nick.fisch@dfo-mpo.gc.ca")
   )
Author: Jon T. Schnute [aut],
  Alex Couture-Beil [aut],
  Rowan Haigh [aut],
  Nicholas Boers [aut],
  Anisa Egeli [aut],
  A. R. Kronlund [ctb],
  Steve Martell [ctb],
  Norm Olsen [ctb],
  Nick Fisch [cre]
Maintainer: Nick Fisch <nick.fisch@dfo-mpo.gc.ca>
Copyright: 2005-2025, Fisheries and Oceans Canada
Depends: R (>= 4.2.0)
Imports: methods, tcltk, XML
Suggests: PBSmapping, deSolve, KernSmooth
NeedsCompilation: yes
SystemRequirements: BWidget
Description: Provides software to facilitate the design, testing, and operation
   of computer models. It focuses particularly on tools that make it easy to
   construct and edit a customized graphical user interface ('GUI'). Although our
   simplified 'GUI' language depends heavily on the R interface to the 'Tcl/Tk'
   package, a user does not need to know 'Tcl/Tk'. Examples illustrate models
   built with other R packages, including 'PBSmapping', 'PBSddesolve', and 'BRugs'. 
   A complete user's guide 'PBSmodelling-UG.pdf' shows how to use this package
   effectively.
License: GPL (>= 2)
URL: https://github.com/pbs-software/pbs-modelling
Packaged: 2025-11-13 16:21:13 UTC; HaighR
Repository: CRAN
Date/Publication: 2025-11-15 12:00:25 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 23:50:44 UTC; unix
Archs: PBSmodelling.so.dSYM
