## For a list of all functions, exported and hidden, see the files 
## PBSMobjects.txt and INDEX in the R libary directory for PBSmodelling.

##classes used in present talk - except they are only internally used
#exportClasses( text, file, code, break, section, talk ) 

exportClasses( PBSoptions )

## Export all functions that do NOT begin with a dot (period)
#exportPattern("^[^\\*]") # export all objects
exportPattern("^[^\\.]")

## Export dot functions by action (explicit list just to keep track)
## Do not export dot functions as CRAN now requires that they be documented (RH 231024)
## Export dot functions because their absence makes their use in reliant packages awkward (RH 241030)
exportPattern("^\\.add")
exportPattern("^\\.auto")
exportPattern("^\\.build")
exportPattern("^\\.cat")
exportPattern("^\\.check")
exportPattern("^\\.clean")
exportPattern("^\\.close")
exportPattern("^\\.convert")
exportPattern("^\\.create")
exportPattern("^\\.d[CU]") # demo Close, Update
exportPattern("^\\.do")
exportPattern("^\\.extract")
exportPattern("^\\.fib")
exportPattern("^\\.find")
exportPattern("^\\.force")
exportPattern("^\\.get")
exportPattern("^\\.gui")
exportPattern("^\\.in")
exportPattern("^\\.load")
exportPattern("^\\.make")
exportPattern("^\\.map")
exportPattern("^\\.matrix")
exportPattern("^\\.max")
exportPattern("^\\.merge")
exportPattern("^\\.next")
exportPattern("^\\.options")
exportPattern("^\\.pack")
exportPattern("^\\.parse")
exportPattern("^\\.pFormat")
exportPattern("^\\.present")
exportPattern("^\\.prev")
exportPattern("^\\.read")
exportPattern("^\\.regex")
exportPattern("^\\.remove")
exportPattern("^\\.search")
exportPattern("^\\.section")
exportPattern("^\\.select")
exportPattern("^\\.set")
exportPattern("^\\.show")
exportPattern("^\\.slide")
exportPattern("^\\.sort")
exportPattern("^\\.start")
exportPattern("^\\.stop")
exportPattern("^\\.strip")
exportPattern("^\\.super")
exportPattern("^\\.table")
exportPattern("^\\.tag")
exportPattern("^\\.tcl")
exportPattern("^\\.trim")
exportPattern("^\\.try")
exportPattern("^\\.update")
exportPattern("^\\.validate")
exportPattern("^\\.view")
exportPattern("^\\.widget")
exportPattern("^\\.win")  ## GUI function wrappers
exportPattern("^\\.write")

 ## need .PBSmodEnv
export(.PBSmodEnv)
export(.PBSdimnameHelper) ## almost frivolous

## Packages declared in the 'Depends' field should not also be in the 'Imports' field (from R-exts.pdf).
## Import all functions from the package tcltk, which is heavily used by PBSmodelling

import(methods, tcltk, XML)

## Imports for undefined globals:

importFrom("graphics", "abline", "arrows", "axis", "box", "contour",
  "frame", "hist", "legend", "lines", "mtext", "pairs", "par", "plot",
  "plot.new", "points", "polygon", "rect", "segments", "strwidth",
  "symbols", "text")

importFrom("grDevices", "col2rgb", "colorRamp", "colors", "contourLines",
  "dev.cur", "dev.new", "dev.off", "extendrange", "grey", "rgb")

importFrom("stats", "acf", "approx", "cor", "density", "median", "nlm",
  "nlminb", "optim", "qnorm", "rnorm", "runif", "update")

importFrom("utils", "browseURL", "capture.output", "demo", "fix",
  "make.packages.html", "str", "vignette")

