The window description files in this directory provide samples of all 
widgets available in PBS Modeling.

The function testWidgets() allows you to view GUIs created with simple 
code written according to specifications in Appendix A of the User's 
Guide.* You can also experiment to discover how minor code changes alter 
a GUI's appearance.

-------------------------------------------------------------------------

*Note: Please remember to consult the User's Guide contained in the file

  ...\library\PBSmodelling\PBSmodelling-UG.pdf,

where "..." denotes the path to your R installation.
