ChangeLog.txt             Incremental changes made to PBSmodelling 
                          during each version.

DeveloperGuide.pdf        Overview of internal data-structures and 
                          algorithms used in PBS Modelling.

PBSmodellingIntro.pdf     Vignette introducing the broad scope of 
                          the PBSmodelling package.

PBSmodelling-UG.pdf       Detailed User's Guide to this package with more
                          information than that contained in the help files.

PBSmodelling-Updates.pdf  Changes to the package not yet covered in the 
                          comprehensive User's Guide. This file may be 
                          absent if the Guide is current.
